;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;          These are some useful general (utility) routines:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                mvsetq
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Description for MVSETQ:
;;
(defmacro mvsetq (x y)
  "
;; Description:

Mvsetq (multiple-value-setq) initializes several variables at once.
;; Examples:
 (mvsetq (a b) (3 666))
 (help multiple-value-setq)
"
  `(multiple-value-setq ,x (values ,@y))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                mvsq
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun mvsq(y &optional z)
  "
;; Description: mvsq(y &optional z)

    This form of multiple-value-setq works even if you use mapcar: so if you
have a bunch of variables named in a list variables, and a bunch of stuff you
want it set to (in list stuff), then

	(mvsq variables stuff)

will work. Or you can do it one variable at a time, as in the examples below.

;; Notes:

	If you have missing values in a list indicated by a '., then you will
get into trouble. You can turn these into vectors, and all will work fine. Not
sure why this is.

;; See Also: genvars

;; Examples:

 (mvsq '(sex age) '(\"M\" 10))
 (mvsq '(\"sex\" \"age\") '(\"M\" 10))
 (mvsq '((\"sex\" \"M\") (\"age\" 10)))
 (mvsq '((\"hi\" (1 2 3))))
 (mvsq '(\"hi\") '((1 2 3 4)))
 (mvsq (list (list \"sex\" \"M\") (list \"age\" (id 3))))
 ;; in the following case, the list of lists will be changed to an array of
 ;; lists: this is a problem with genvars.
 (mvsq (list \"sex\" \"Mouthwash\") (list '((1 2) (3 4)) (id 3)))
"
  (if z (setq y (weave y z)))
  (dolist
   (x y)
   (let ((stuff (second x)))
     (if (stringp (first x))
	 (genvars (first x) stuff)
       (genvars (symbol-name (first x)) stuff)
       )
     )
   )
  (values)
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                grid
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun grid (a b n)
  "
;; Description:
 (grid a b n)

See rseq. Grid handles the special case where one wants a single step, but uses
a step-size of (b-a)/n in going from a to b. I use it for left and right
rectangle rules for integration.

Roughly translates as \"go from a to b in n steps\". It's length is one more
than the number of steps:

;; Examples:
 (grid 0 1 10)
 (length (grid 0 1 10))
"
  (if (= 1 n) (list a b)
    (rseq a b (1+ n))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                paws
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun paws (nsecs)
  "Args: nsecs - number of seconds to pause.
Uses the pause function. Example:
		(paws 10)
to pause for 10 seconds.

;; Examples:
 (pause 5)
"
  (pause (* 60 nsecs))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                element
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun element (x nel)
  "
;; Description: function (element x nel):

Args: x, a list; nel the index of the element.

Remember that lists are indexed from 0.

;; Examples:
 (def x (list 1 2 3))
 (element x 2)		;; 3
 (element x (list 0 2))	;; (1 3)		
"
  (select x nel)
)
;;                                
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                firsts
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun firsts (x)
"
;; Gets the \"even\" elements of a list

;; See Also: seconds, nth-ones

;; Examples:
 (firsts (iseq 7))
 (seconds (iseq 7))
"
  (select x 
	  (* 2
	     (butlast (iseq 0 (ceiling (/ (length x) 2))))
	     )
	  )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                seconds
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun seconds (x)
"
;; Gets the \"odd\" elements of a list

;; See Also: firsts, nth-ones

;; Examples:
 (firsts (iseq 7))
 (seconds (iseq 7))
"
  (select (rest x)
	  (* 2
	     (butlast (iseq 0 (ceiling (/ (length (rest x)) 2))))
	     )
	  )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                nth-ones
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun nth-ones (x n)
  "
;; Generalizes firsts and seconds. Firsts is (nth x 2), and seconds is 
;; (nth (rest x) 2).

;; Gets every nth element from a list.

;; See Also: firsts, seconds

;; Examples:
 (nth-ones (iseq 7) 3)
 (firsts (iseq 7))
 (nth-ones (iseq 7) 2)
 (seconds (iseq 7))
 (nth-ones (rest (iseq 7)) 2)
"
  (select x 
	  (* n
	     (butlast (iseq 0 (ceiling (/ (length x) n))))
	     )
	  )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; abscissa Doc:    Creation Date:12/31/95        Author: Andy Long
;;
;;
(defun abscissa(data.list)
  "
;; Description/Arguments:

       (abscissa data.list)
  
       This pulls the first coordinate of pairs of coordinates off a data list
 (that is, a list of ordered pairs)

;; Notes:

;; See Also: 

;; Examples:
  (abscissa '((0 1) (1 2) (2 3)))	;; (0 1 2)
  (abscissa #2a(((1 3) (2 5))))		;; #2A((1 2))
"
  (if (matrixp data.list)
      (let ((size (size data.list)))
	(matrix size (mapcar #'first (matrix-to-list data.list)))
	)
    (mapcar #'first data.list)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; ordinates Doc:    Creation Date:12/31/95        Author: Andy Long
;;
;;
(defun ordinates(data.list)
  "
;; Description/Arguments:

       (ordinates data.list)
  
Returns the list of the second coordinates of a list of ordered whatevers.

;; Notes:

;; See Also: xs, abscissa, ys

;; Examples:
  (ordinates '((0 1) (1 2) (2 3)))	;; (1 2 3)
  (ordinates #2a(((1 3) (2 5))))	;; #2A((3 5))
"
  (if (matrixp data.list)
      (let ((size (size data.list)))
	(matrix size (mapcar #'second (matrix-to-list data.list)))
	)
    (mapcar #'second data.list)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; zs Doc:    Creation Date:12/31/95        Author: Andy Long
;;
;;
(defun zs(data.list)
  "
;; Description/Arguments:

       (zs data.list)
  
Returns the list of the third coordinates of a list of triplets.

;; Notes:

;; See Also: xs, ys

;; Examples:
  (zs '((0 1 3) (1 2 6 ) (2 3 7)))	;; (3 6 7)
  (zs #2a(((1 3 4) (2 5 7))))		;; #2A((4 7))
"
  (if (matrixp data.list)
      (let ((size (size data.list)))
	(matrix size (mapcar #'third (matrix-to-list data.list)))
	)
    (mapcar #'third data.list)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; echo Doc:    Creation Date://        Author: Andy Long
;;
;;
(defun echo(x)
  "
;; Description:

        (echo x)
  
Just echos back the argument: a useful way to return values from a macro.

;; Arguments

;; See Also:

;; Examples:
 (echo 1)
 (def a 1)
 (echo a)
"
  `,x
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; message Doc:    Creation Date:1/22/96        Author: Andy Long
;;
;;
(defun message (str1)
  "
;; Description/Arguments:

        (message str1)

	Prints your message, then waits for a return key (or any other key,
essentially). Mostly useful for scripts: otherwise, the return to start
a new line will trip it. 

;; Returns   ...

;; Notes:

;; See Also: princ, read-char

;; Examples:
 (message \"hi\")
 (message \"Your function is...\")
"
  (format t "~a~%Hit return to continue:" str1)
  (read-line)
  '...
  )
(defun message-execute (x)
  ;; write the input to file stuff.ola:
  (with-open-file
   (f "stuff.ola" :direction :output)
   (format f "~a" x)
   )
  ;; Show the input:
  (format t "~a" x)
  (message-dialog x :strict t)

  ;; Let's have a look at the output:
  (load "stuff.ola" :print t)
  (force-output)

  ;; clean up:
  (delete-file "stuff.ola")

  ;; wait before going on:
  (format t "~%Hit return to continue:")
  (read-line)
  )

(defun message-execute-only (x)
  ;; write the input to file stuff.ola:
  (with-open-file
   (f "stuff.ola" :direction :output)
   (format f "~a" x)
   )
  ;; Show the input:
  (format t "~a" x)
  (message-dialog x :strict t)

  ;; Let's have a look at the output:
  (load "stuff.ola" :print t)
  (force-output)

  ;; clean up:
  (delete-file "stuff.ola")
  )

;; ael: needed to redefine message-dialog to allow for strict passing of
;; argument (rather than passing it to format first):
(defun message-dialog (&rest args)
  "Args: (s &rest args)
Open modal dialog with string S and OK buttons. Returns NIL. S can contain
format directives, which are filled from the remaining arguments."
  (let* (
	 (strict (first (catch-keys '(:strict) args)))
	 (args (remove-keys '(:strict) args))
	 (d (send message-dialog-proto :new 
		  (if strict args (apply #'format nil args)))))
    (send d :modal-dialog)))
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; pick Doc:    Creation Date:1/30/96        Author: Andy Long
;;
;;
(defun pick(n a &optional (replace nil))
  "
;; Description/Arguments:

        (pick n a &optional replace nil)
  
Picks n elements from list a, with replacement. A feeble approximation to
sample, written before I knew it existed.

;; Notes: relies on function sample

;; See Also: random-elt, sample, sort-data

;; Examples:
 (pick 2 '(2 7 9 10 13))
 (pick 6 (iseq 1 52) t) ;; with replacement
"
  (sort-data (sample a n replace))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; gnuplot Doc:    Creation Date:2/3/96        Author: Andy Long
;;
;;
(defun gnuplot()
  "
;; Description/Arguments:

        (gnuplot)
  
;; Notes:

       Prepares a plot for gnuplot, which can then turn it into a postscript
file, say.

;; See Also: :to-gnuplot

;; Examples:

"
  (send (first (active-graph-windows)) :to-gnuplot)
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			Some System Commands:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun up () 
  "up: Go up one directory in the directory tree."
  #+macintosh (set-working-directory "::")
  #-macintosh (set-working-directory ".."))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                cd
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro cd (x)
  "
Description: macro cd changes directory

;; Examples:
;;  (cd math)
 (setq working-directory (get-working-directory))
 #+macintosh (cd \"::\")
 #-macintosh (cd \"..\")
 (eval `(cd ,working-directory))
 ;; use a string if your directory structure is case sensitive (e.g. UNIX):
;; (cd \"Math\") 
"
  (let (
        (xx (if (eval (stringp `,x)) `,x
              (nstring-downcase (symbol-name `,x))))
	)
    (if (or (equalp xx "..") (equalp xx "::")) (up)
      (if (equalp (elt xx 0) #\/)
	  (set-working-directory (strcat xx separator))
	#+unix (set-working-directory (strcat (cwd) separator xx separator))
	#+macintosh (set-working-directory (strcat (cwd) xx))
	)
      )
    )
  (cwd)
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
(defun home ()
  "
;; Description: (home)
     Sets the current working directory to that in *default-path*.

;; Examples:
 (home)
"
  (set-working-directory *default-path*)
  (cwd)
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
(defmacro calculus (&optional cdir)
  "
;; Description: (calculus &optional cdir)
     Sets the current working directory to that in *andy-default-path* (the
calculus stuff). Optional argument cdir will get one to a subdirectory of that
calculus directory.

;; Examples:
 (calculus)
 (calculus math)
 (calculus etc)
"
  (set-working-directory *andy-default-path*)
  (if cdir (eval `(cd ,cdir)) (cwd))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; about Doc:    Creation Date:3/4/96        Author: Andy Long
;;
;;
(defmacro about(topic)
  "
;; Description/Arguments:

       (about topic)
  
;; code:
   `(apropos ',topic)

;; Notes:

;; See Also:

;; Examples:
 (about help)
"
  `(apropos ',topic)
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; again Doc:    Creation Date:3/29/96        Author: Andy Long
;;
;;
(defun again (&optional (command +))
  "
;; Description/Arguments:

       (again &optional (command +))
  
repeats the previous command.

;; Notes: I wish that I had a history mechanism, more than just the three
previous commands....

;; See Also: +, ++, +++, his, history
"
  (if (listp command)
      (if (eq (first command) 'AGAIN)
	  (if history (history (length history)))
	(let()
	  (his)
	  (eval command)
	  )
	)
    (eval command)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                try-out
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro try-out (routine-name &key proto)
  "
;; Description: (try-out routine-name &key proto)

	This will read in the documentation for a command, and if there is an
example or two (signalled by the presence of a line in the documentation
with two semi-colons followed by  'Example' as below) then the examples will
be run.

	If you want to try-out a method of a proto, then use the :proto key, as
shown in the example below for :adjust.

;; Examples:
 (try-out plot)
 (try-out :adjust :proto regression-model-proto)
"
  `(helplass ',routine-name :proto ,proto)
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                helplass
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun helplass (&optional s &key proto)
  "
;; Description - this is a helper for macro try-out.

		(helplass s &key proto string)

;; You can use it directly to try a command, using the examples in the
;; documentation:

;; Examples:
 (helplass 'func)
 (helplass :adjust :proto regression-model-proto)
 (helplass
   \"
;; Examples
  (plot tan)
  \")
"
  (cond 
   ((null s)
    (princ "No lass is good lass?! Helplass needs an argument, but you can't
get one without a lass."))
    
   ((stringp s)
    #| - ael - Tue Apr 28 22:22:51 1998
    (print (coerce (grep-replace (list #\newline) (list #\space) (coerce s 'vector))
	     'string))
     (coerce (grep-replace (list #\newline) (list #\space) (coerce s 'vector))
	     'string)
 - lea - |#
    (break-string-into-lines-and-evaluate s ))
    
   (proto
    (break-string-into-lines-and-evaluate (send proto :documentation s)))
    
   (t 
    (dolist
     (i 
      (let* (
	     (fname
	      (strcat
	       *andy-default-path*
	       "Help" separator (string-downcase (symbol-name s))))
	     (help-file (open fname :direction :probe))
	     )
	(if help-file ;; there's a help file, which is hopefully of the
		      ;; standard form, that will have some examples to try:
	    (list 
	     (with-output-to-string
	      (f)
	      (mapcar #'(lambda(x) (format f "~a~%" x)) (read-file fname)))
	     )
	  ;; otherwise, we'll start with usual documentation stuff...
	  (list
	   (documentation s 'function)
	   (documentation s 'variable)
	   (documentation s 'type)
	   (documentation s 'setf)
	   ))
	))
     (if i (let ()
	     (princ i)
	     (break-string-into-lines-and-evaluate i)
	     )
       )
     )
    )
   )
  (values)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;               break-string-into-lines-and-evaluate (string)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun break-string-into-lines-and-evaluate (string)
  "
       Function (break-string-into-lines-and-evaluate string)

       Takes a string containing new-line characters (i.e., carriage returns)
and writes it as a list of strings (the individual lines). If there is a line
starting with \";; Example\", then the results will begin to be evaluated.

;; See also: remove-comments

;; Examples:
 (break-string-into-lines-and-evaluate 
 \"
Hi, how's it
going?
;; Examples:
 (iseq 10)
\")
"
  (let (
	(inputs (make-string-input-stream string))
	(j 0)
	(lines-of-code '(list))
	string-of-lines
	long-string
	evaluate-em
	)
    (dotimes
     (i (length string))
     (if (eq #\Newline (read-char inputs)) (setq j (1+ j))))
    (setq inputs (make-string-input-stream string))
    (dotimes
     (i j)
     (let* ((line (space-trim (read-line inputs))))
       (setq string-of-lines (combine string-of-lines line))
       (if (not evaluate-em)
	   ;; The following line searches for the tag (;; Example) which
	   ;; indicates that executible stuff is to follow.
	   (if (search ";; Example" line)(setq evaluate-em t))
	 (if (/= (length line) 0)
	     (setq
	      long-string
	      (strcat
	       long-string " "
	       (remove-comments line)
	       )
	      )
	   )
	 )
       )
     )
    (andy-print long-string)
    (dolist
     (i (string-to-list long-string))
     (print (eval i))
     )
    nil
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 remove-comments
;;
(defun remove-comments(line)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          remove-comments Doc:
;;
;; Author: Andy Long
;; Created: Wed Oct 22 15:49:41 1997
;;
;; Description/Arguments:

       (remove-comments line)
  
	where line is a line of text (string).

Remove-comments checks for semi-colons (that haven't been escaped). Note that
it requires three back-slashes to escape the semi-colon (the last of the
following examples).

;; Notes: uses subseq rather than select, for backward compatibility: in 3.44
;; you couldn't use select on a string.

;; Notes: wrote this as a helper to break-string-into-lines-and-evaluate, which
;; is used in try-out.

;; See Also: break-string-into-lines-and-evaluate

;; Examples:
 (remove-comments \"Hi there: how's it going?\") ; this is a comment
 (remove-comments \"Hi there: how's it going?\\; comment\")
 (remove-comments \"Hi there: how's it going?\\\\; comment\")
 (remove-comments \"Hi there\\\\\\; how's it going?\") 
"
  (let ((i (search ";" line)))
    (if i
	(let ((j (search "\\\;" line)))
	  (if j
	      (if (= (- i j) 1)
		  (strcat (subseq line 0 j)
			  (subseq line i (length line)))
		(subseq line 0 i)
		)
	    (subseq line 0 i)
	    )
	  )
      line)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                       break-string-into-lines(string)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun break-string-into-lines(string)
  "
       Function (break-string-into-lines string)

Takes a string containing new-line characters (i.e., carriage returns) and
writes it as a list of strings (the individual lines).

;; Examples:
 (break-string-into-lines
 \"
Hi, how's it
going?
;; Examples:
 (iseq 10)
\")
"
  (let (
	(inputs (make-string-input-stream string))
	(string-of-lines nil)
	(j 0)
	)
    (dotimes (i (length string))
	     (if (eq #\Newline (read-char inputs)) (setq j (1+ j))))
    (setq inputs (make-string-input-stream string))
    (dotimes (i j)
	     (let* (
		    (line (read-line inputs))
		    )
	       (setq string-of-lines (combine string-of-lines line))
	       )
	     )
    (rest string-of-lines)
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                is-it-bound
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro is-it-bound (i)
  "
;; Description: a macro for boundp:

;; Examples:
 (is-it-bound aaa)
 (def aaa 3)
 (is-it-bound aaa)
"
  (if (boundp `,i) t nil))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; cls Doc:    Creation Date:4/21/96        Author: Andy Long
;;
;;
#+macintosh
(defun cls()
  "
;; Description/Arguments: (cls)

       cls flushes the listener window (and you lose your 'history').
  
;; Notes: Here's the code: (send *listener* :flush-window)

;; Examples:
 (cls)
"
  (send *listener* :flush-window)  
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; string-to-list:    Creation Date:5/22/95        Author: Andy Long
;;
;;
(defun string-to-list (string)
  "
;; Description: (string-to-list string)

Turns your string into a list. If you have an empty string, i.e. \"\", it
returns nil.

;; Examples:
 (string-to-list \"hi how many 88\")
 (string-to-list \"1 (2 3) 3\")
 (string-to-list \"\")
 (string-to-list \"	3\")
;; but not
;; (string-to-list \"4:30\")
;; (string-to-list \"(4\:30)\")
;; (string-to-list \"hi how many 88?\")
;; (string-to-list \"6e\") ; returns (6) - interprets it as 6e0, scientific
;; 				notation.
"
  (read-from-string (strcat "(" string ")"))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; v-and Doc:    Creation Date:5/22/95        Author: Andy Long
;;
;;
(defun v-and (n)
  "
;; Description/Arguments: (v-and list-o-statements)

 vectorized and
  
;; Notes:

;; See Also:

;; Examples:
 (v-and '(t t nil)) ;; false
 (v-and '(t (= .5 (/ 1 2)) (< 2 3))) ;; true
"
   (eval `(and ,@n))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                mult-setq
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro mult-setq (aa bb)
  "
;; Description: macro

		(mult-setq list-of-symbols list-of-numerical-values)

 (mult-setq (c d) (1 2))
 c
 d
 (mult-setq (c d e) (rand.u 3))
 c
 d
 e
 ;; but not
 ;; (mult-setq (a b) (c d))
"
  (let (
	(a (eval `(turn-it-into-a-list ',aa)))
	(b (eval `(turn-it-into-a-list ,bb)))
	)
    (dotimes (i (length a))
	     (let (
		   (ai (elt a i))
		   (bi (elt b i))
		   )
	       (eval `(setq ,ai ,bi))
	       )
	     )
    )
  nil
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                printall
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun printall (&rest e)
  "
;; Description: macro printall prints a bunch of things:

;; Examples:
 (printall 1 2 'hi \"hi\")
"
  `,(mapcar #'print e)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                string-to-num
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun string-to-num (x)
  "
;; Examples:
 (string-to-num \"2\")
"
  (read (make-string-input-stream x))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; vcoerce Doc:    Creation Date:6/11/96        Author: Andy Long
;;
;;
(defun vcoerce(v type)
  "
;; Description/Arguments:

       (vcoerce v type)
  
;; Notes:

;; See Also:

;; Examples:
 (def a (vector 1.2 2 3.3 4))
 (vcoerce a 'complex)
 (def a (vector \"1.2\" \"2\" \"3.3\" \"4\"))
 (vcoerce a 'float)
"
  (if (and (eq type 'float) (eq (type-of (elt v 0)) 'string))
      (mapcar #'string-to-num (coerce v 'list))
    (mapcar #'(lambda(x) (coerce x type)) (coerce v 'list))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                neq - not equalp
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun neq (a b)
  "
Just plain ol' not-equalp!

;; Examples:
 (neq 2 'hi)
 (neq 3.1415 pi)
 (neq pi pi)
"
  (if (equalp a b) nil t))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; final Doc:    Creation Date:6/23/96      Author: Andy Long
;;
;;
(defun final(x)
  "
;; Description/Arguments:

       (final x)
  
This is the last element of the list x (or x, if x is not a list).

;; Notes: same as (first (last x)), essentially.

;; See Also: first, last

;; Examples:
 (final '(1 2 3))
"
  (if (listp x) (first (last x)) x)
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; eltt Doc:    Creation Date:6/23/96        Author: Andy Long
;;
;;
(defmacro eltt(aa i)
  "
;; Description/Arguments: macro eltt

       (eltt a(i))
  
is a macro to make some code a little easier to convert from languages other
than xlispstat.

;; Notes:

;; See Also:

;; Examples:
 (setq a '(1 2 3))
 (eltt a(2))
"
  `(elt ',a ,@i)
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; load-calc Doc:    Creation Date:7/6/96        Author: Andy Long
;;
;;
(defmacro load-calc(filename &optional (dir 'math) (print nil))
  "
;; Description/Arguments: macro load-calc

       (filename &optional (dir \"math\") (print nil))
  
   Loads the .lsp file filename.lsp in the directory math (by default) under
the calculus directory. Can also load from the other sub-directories, using the
optional argument.

;; Notes: Use the symbols, unless you have a case sensitive filename, in which
;; case you should use strings. The symbol names will be changed to lower case.

;; Examples:
 (load-calc math)
 (load-calc \"math\") ;; useful if your system (e.g. UNIX) is case sensitive.
 (load-calc utility etc)
"
  (let* (
	 (file
	  (eval (if (stringp `,filename) `,filename 
		  (nstring-downcase (format nil "~a" `,filename)))))
	 (dir
	  (eval (if (stringp `,dir) `,dir 
		  (nstring-downcase (format nil "~a" `,dir)))))
	 (prefix (strcat *default-path* "calculus" separator))
	 (file (strcat separator file))
	 (suffix (if (grep ".lsp" file) file (strcat file ".lsp")))
	 (dir
	  (cond
	   ((probe-file (strcat prefix dir suffix))
	    dir
	    )
	   ((probe-file (strcat prefix "etc" suffix))
	    "etc"
	    )
	   ((probe-file (strcat prefix "hulp" suffix))
	    "hulp"
	    )
	   ((probe-file (strcat prefix "loop" suffix))
	    "loop"
	    )
	   ((probe-file (strcat prefix "graphics" suffix))
	    "graphics"
	    )
	   ((probe-file (strcat prefix "contrib" suffix))
	    "contrib"
	    )
	   ((probe-file (strcat prefix "examples" suffix))
	    "examples")
	   ((probe-file (strcat prefix "modules" suffix))
	    "modules")
	   (t
	    (format t "Looks like trouble - can't find ~s!~%" file)
	    nil
	    )
	   )
	  )
	 )
    (if dir (load (strcat prefix dir suffix) :print print))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; cd-calc Doc:    Creation Date:7/6/96        Author: Andy Long
;;
;;
(defmacro cd-calc(&optional (dir 'math))
  "
;; Description/Arguments: macro cd-calc

       (&optional (dir math))
  
   Changes directory to the math subdirectory of the calculus directory.
the calculus directory.

;; Notes: Use the symbols, unless you have a case sensitive directory, in which
;; case you should use strings.

;; Examples:
 (cd-calc) ;; takes you to the math subdirectory
 (cd-calc etc)
"
  (let (
	(dir
	 (eval (if (stringp `,dir) `,dir 
		 (nstring-downcase (symbol-name `,dir)))))
	)
    (set-working-directory
     (strcat *default-path* "calculus" separator dir))
    (cwd)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; weave Doc:    Creation Date:7/7/96        Author: Andy Long
;;
;;
(defun weave(&rest lists)
  "
;; Weave weaves together a bunch of lists, to make \"tuples\".

;; Examples:
 (weave (iseq 4) (sample (iseq 4) 4))
 (weave (iseq 4) (sample (iseq 4) 4) (rseq 0 2 4))
"
  (transpose lists)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; smoosh Doc:    Creation Date:7/7/96        Author: Andy Long
;;
;;
(defun smoosh(list)
  "
;; Description:

   Smoosh is designed to do what combine doesn't: it will
combine things only one list deep. Compare the examples below:

;; Examples:
 (smoosh (list 3 (iseq 4) (sample (iseq 4) 4)))
 (combine (list 3 (iseq 4) (sample (iseq 4) 4)))
 (smoosh (list 3 (list (iseq 4) (sample (iseq 4) 4))))
 (combine (list 3 (list (iseq 4) (sample (iseq 4) 4))))
"
  (let (new)
    (dolist
     (i list)
     (setq new
	   (append
	    new
	    (if (listp i) (select i (iseq (length i))) (list i))
	    )
	   )
     )
    new
    )
  )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                    round-down
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun round-down (x)
  "
;; Description: round-down (x)
Rounds to next integer closer to 0.

;; Name: Same as truncate

;; Examples:
 (round-down 3.7)
 (round-down -3.7)

;; Note: same as truncate.
"
  (truncate x)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                    round-up
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun round-up (x)
  "
;; Description: round-up (x)
Rounds to next integer farther from 0.

;; Examples:
 (round-up 3.7)
 (round-up -3.7)
"
  (if (< x 0) (floor x) (ceil x))
  )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; 				sgw
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun sgw()
  "
;; Description:
	saw stands for 'show graphics windows'. Just pops all graph-proto
windows into view.
"
  (mapcar #'(lambda(x) (send x :show-window t)) (agw))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			read-matrix-data
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun read-matrix-data (file &optional ncols)
  "
;; Description: read-matrix-data (file &optional ncols)

	Reads a data file as a matrix.

 (read-matrix-data \"geo.dat\")
"
  (let* (
	 (columns
	  (if ncols
	      ncols
	    (let* ((f (open file))
		   (ncols (length (string-to-list (read-line f))))
		   (f (close f)))
	      ncols
	      )	      
	    )
	  )
	 (long-line (read-data-file file))
	 )
    (matrix (list (/ (length long-line) columns) columns) long-line)
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			read-matrix-with-dimensions
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun read-matrix-with-dimensions (file nrows ncols)
  "
;; Description: read-matrix-with-dimensions (file nrows ncols)

	This was written just so that I could read in a matrix as a matrix,
rather than read it in as a list and convert it to a matrix. I had memory
problems, and could read in a big list as a list, but couldn't then convert it
to a matrix (too many nodes).

;; This should be eliminated: read-data-columns already exists.

;; Examples:
 (help read-data-columns)

"
  (let (
	(mat (make-array (list nrows ncols)))
	(f (open file))
	)
    (dotimes
     (i nrows)
     (let ((row (string-to-list (read-line f))))
       (dotimes
	(j ncols)
	(setf (aref mat i j) (elt row j))
	)
       )
     )
    (close f)
    mat
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			re-standardize
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun re-standardize(x mean sd)
  "
;; Description: re-standardize(x mean sd)

	A variable that has been standardized is retransformed
using re-standardize.

;; Examples:
 (def stuff (rand.u 100))
 (def staff (standardize stuff))
 (max (- stuff (re-standardize staff (mean stuff) (sd stuff)))) ;; 0?
 (min (- stuff (re-standardize staff (mean stuff) (sd stuff)))) ;; 0?
"
  (+ mean (* sd x))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			map-it
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro map-it(func-form lis)
  "
;; Description: map-it(func-form list)

	map-it just saves you the strokes involved in using a lambda
expression. map-it is a macro, which means that you have to be a little
careful using it in a function. Here's the code:

  `(mapcar #'(lambda(x) ,func-form) ,lis)

so there's not much point in using it in functions anyway!

;; See also: {}

;; Examples:
 (map-it (sin x) (rseq 0 pi 10))
 (plot (map-it (sin x) (rseq 0 pi 100)))

 ;; maybe you'd rather use braces:
 (plot {sin x} 0 pi)
"
  `(mapcar #'(lambda(x) ,func-form) ,lis)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			factor
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun factor(i)
  "
;; Description: 

	Prime factorization of an integer.

;; Examples:
 (factor 1)
 (factor 32)
 (factor 839) ;; prime
 (factor 120)
 (factor 1200)

;; (1) 
;; (2 2 2 2 2) 
;; (839) 
;; (2 2 2 3 5) 
;; (2 2 2 2 3 5 5) 
"
  (let (
	quo
	factors
	)
    (tagbody
     (dolist
	 (j (rest (rest (iseq (floor (/ i 2))))))
       (setq quo (/ i j))
       (if (= quo (floor quo))
	   (let ()
	     (setq factors (append factors (list j)))
	     (go done)
	     )
	 )
       )
     (setq factors 
	   (append factors 
		   (if (= i 4) (list 2 2) (list i))))
     (go finished)
     done
     (if quo (if (/= quo 1)
		 (setq factors (append factors (factor quo))))
       (setq factors (append factors (list i)))
       )
     finished
     )
    factors
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          conversion formulas....
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def *conversions* '(kg-lb lb-kg ft-in-cm cm-ft-in))
(defun kg-lb(x) "kg-lb(x)" (* x 2.2046))
(defun lb-kg(x) "lb-kg(x)" (/ x 2.2046))
(defun ft-in-cm (x y) "ft-in-cm (x y)" (* 2.54 (+ (* x 12) y)))
(defun cm-ft-in (x) "cm-ft-in (x)"
  (let ((inches (/ x 2.54)))
    (list (floor (/ inches 12)) (round (- inches (* 12 (floor (/ inches 12))))))
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                x-y-rotation
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun x-y-rotation (alpha)
  "
;; Description: (x-y-rotation alpha)

	produces the 3x3 rotation matrix giving a rotation of the xy-plane by
an angle of alpha *clock-wise*.

;; Examples:
 ;; The following rotates the x unit vector to the 45 degree line of the 
 ;; fourth quadrant:
 (matprod (x-y-rotation (/ pi 4)) '(1 0 0))
"
  (let (
	(c (cos alpha))
	(s (sin alpha))
	)
    (matrix '(3 3) (list c s 0 (- s) c 0 0 0 1))
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                x-z-rotation
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun x-z-rotation (alpha)
  "
;; Description: (x-z-rotation alpha)

	produces the 3x3 rotation matrix giving a rotation of the xz-plane by
an angle of alpha *clock-wise*.

;; Examples:
 ;; The following rotates the x unit vector to the 45 degree line of the 
 ;; fourth quadrant:
 (matprod (x-z-rotation (/ pi 4)) '(1 0 0))
"
  (let (
	(c (cos alpha))
	(s (sin alpha))
	)
    (matrix '(3 3) (list c 0 s 0 1 0 (- s) 0 c))
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                y-z-rotation
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun y-z-rotation (alpha)
  "
;; Description: (y-z-rotation alpha)

	produces the 3x3 rotation matrix giving a rotation of the yz-plane by
an angle of alpha *clock-wise*.

;; Examples:
 ;; The following rotates the x unit vector to the 45 degree line of the 
 ;; fourth quadrant:
 (matprod (y-z-rotation (/ pi 4)) '(0 1 0))
"
  (let (
	(c (cos alpha))
	(s (sin alpha))
	)
    (matrix '(3 3) (list 1 0 0 0 c s 0 (- s) c))
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                rotation
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun rotation (alpha beta gamma)
  "
;; Description: rotation (alpha beta gamma)

	rotation produces the 3x3 rotation matrix corresponding to the Euler
angles alpha beta and gamma.

;; Examples:
 (mprint (rotation .3 .4 .5))
 (determinant (rotation .3 .4 .5)) ;; should be 1: rotation matrix!
"
  (matmult (x-y-rotation alpha) (y-z-rotation beta) (x-y-rotation gamma))
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                quit
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun quit()
  "
;; Description: same as (exit). Gets you the heck out!
"
  (exit)
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                            print-every
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun print-every(n k &optional something)
  "
;; Documentation: (print-every n k &optional something)

;; Examples:
 (dotimes (i 100) (print-every 10 i 'hi)) ;; prints HI every ten iterations.
 (dotimes (i 100) (print-every 10 i)) ;; prints i every ten iterations.
"
  (if (= (/ k n) (floor (/ k n))) (print (if something something k)))
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                            grid-over-data
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun grid-over-data(x y &key (n1 20) (n2 20))
  "
;; Documentation: (grid-over-data x y &key (n1 20) (n2 20))

	Creates a list of pairs of points which form a grid encompassing a
two-dimensional data set.

;; Examples:
 (new-plot)
 (lines off)
 (def xgrid (rand.u 100))
 (def ygrid (rand.n 100))
 (plot (grid-over-data xgrid ygrid))
 (send my-plot :add-plot-item (list 'points xgrid ygrid 'blue 'diamond))
"
  (let (
	(xmin (min x))
	(xmax (max x))
	(ymin (min y))
	(ymax (max y))
	)
    (weave
     (repeat (rseq xmin xmax 20) 20)
     (combine (mapcar #'(lambda(x) (repeat x 20)) (rseq ymin ymax 20)))
     )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                            catch-multiple-values
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro catch-multiple-values(x)
  "
;; Documentation: macro (catch-multiple-values x)

	Simply calls multiple-value-list with the arguments x.

;; See also: multiple-value-list and the alias cmv (who wants to type out
;; catch-multiple-values?)

;; Examples:
 (catch-multiple-values (quadratic 1 0 -1 (rseq 0 1 10)))
"
  `(multiple-value-list ,x)
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                subset
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun subset(x y)
  "
;; Description: (subset x y)

subsets x by the values of y. Returns a pair of lists: the first is the
different values of y, and the second is a list of the corresponding lists.

;; Notes: uses remove-duplicates, simul-keep-if

;; Examples:
 (subset '(1.0 2.2 3.5) '(1 0 1))
 ;; returns ((0 1) ((2.2) (1.0 3.5)))   
 (subset '(1.0 2.2 3.5) '(m v m))
 ;; returns ((V M) ((2.2) (1.0 3.5)))
"
  (let* (
	 (xx (if (listp x) x (coerce x 'list)))
	 (yy (if (listp y) y (coerce y 'list)))
	 (vals (remove-duplicates yy))
	 results
	 )
    (dolist
     (i vals)
     (setq
      results
      (append results
	      (list (second (simul-keep-if yy xx #'(lambda(z) (eql z i)))))))
     )
    (list vals results)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                            space-trim
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun space-trim(x)
  "
;; Documentation: (space-trim string)

	Space-trim removes spaces from both before and after a string.

;; See Also: string-left-trim, string-right-trim

;; Examples:
 (space-trim \" Howdy! \")
 ;; \"Howdy!\"
"
  (if (listp x)
      (mapcar #'string-trim x)
    (string-right-trim " " (string-left-trim " " x))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; genvars Doc:    Creation Date:5/27/96        Author: Andy Long
;;
;; genvars Doc:    Modification Date:12/18/96        Author: Andy Long
;;
(defun genvars (&rest aa)
  "
;; Description/Arguments:

       (genvars variable-value-pairs)
  
;; Notes: genvars defines one or more variables at a time, using strings for
;; variables names. I wrote this so that I could create many variables from a
;; single variable name (for example, a data set \"data\" could have variables
;; data.title, data.length, etc. automatically defined in a procedure.

;; See Also: split-list, read-from-string

;; Examples:

 (genvars \"a\" 5 \"b\" 6)
 a b
 (genvars \"b\" 5 \"a\" 6)
 a b
 (genvars '(\"b\" 5) '(\"a\" 6))
 a b
 ;; The next two work, but only by changing a list of lists to an array of
 ;; lists, rats! I think that the trouble is in the ,@ function.
 (genvars \"b\" '((1 2) (2 3)))
 b
 (genvars \"b\" '(1 (2 3)))
 b
 (genvars \"b\" #((1 2) (2 3)))
 b
"
  (let ((a (if (listp (first aa)) aa (split-list aa 2))))
    (dolist
     (i a)
     (let (
	   (newvar (read-from-string (first i)))
	   (newval (second i))
	   )
       (if (listp newval)
	   (if (search (list t) (mapcar #'listp newval))
	       (let ((switch (coerce newval 'vector)))
		 (print "list of lists -> array of lists")
		 (eval `(def ,newvar ,switch))
		 )
	     (eval `(def ,newvar (list ,@newval))))
	 (eval `(def ,newvar ,newval))
	 )
       )
     )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; genvarsetq Doc:    Creation Date:5/27/96        Author: Andy Long
;;
;; genvarsetq Doc:    Modification Date:12/18/96        Author: Andy Long
;;
(defun genvarsetq (&rest aa)
  "
;; Description/Arguments:

       (genvarsetq variable-value-pairs)
  
;; Notes: genvarsetq defines one or more variables at a time, using strings for
;; variables names. I wrote this so that I could create many variables from a
;; single variable name (for example, a data set \"data\" could have variables
;; data.title, data.length, etc. automatically defined in a procedure.

;; See Also: split-list, read-from-string

;; Examples:

 (genvarsetq \"a\" 5 \"b\" 6)
 a b
 (genvarsetq \"b\" 5 \"a\" 6)
 a b
 (genvarsetq '(\"b\" 5) '(\"a\" 6))
 a b
 ;; The next two work, but only by changing a list of lists to an array of
 ;; lists, rats! I think that the trouble is in the ,@ function.
 (genvarsetq \"b\" '((1 2) (2 3)))
 b
 (genvarsetq \"b\" '(1 (2 3)))
 b
 (genvarsetq \"b\" #((1 2) (2 3)))
 b
"
  (let ((a (if (listp (first aa)) aa (split-list aa 2))))
    (dolist
     (i a)
     (let (
	   (newvar (read-from-string (first i)))
	   (newval (second i))
	   )
       (if (listp newval)
	   (if (search (list t) (mapcar #'listp newval))
	       (let ((switch (coerce newval 'vector)))
		 (print "list of lists -> array of lists")
		 (eval `(setq ,newvar ,switch))
		 )
	     (eval `(setq ,newvar (list ,@newval))))
	 (eval `(setq ,newvar ,newval))
	 )
       )
     )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 and-function
;;
(defun and-function(&rest x)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          and-function Doc:
;;
;; Author: Andy Long
;; Created: Wed Apr 2 12:03:30 CST 1997
;;
;; Description/Arguments:

       (and-function &rest x)
  
;; Notes: I created this because I couldn't 'apply' and - it's an fsub, which
;; is pretty obstinate!

;; See Also: every

;; Examples:
 (and-function t t)
 (and-function nil t)
 (and-function  nil nil)
 (apply #'and-function (list nil t))
 (mapcar #'and-function (list nil t))
 ;; but can't do this:
 ;; (apply #'and (list nil t))
"
  (every #'(lambda(x) x) x)
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 or-function
;;
(defun or-function(&rest x)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          or-function Doc:
;;
;; Author: Andy Long
;; Created: Wed Apr 2 12:03:30 CST 1997
;;
;; Description/Arguments:

       (or-function &rest x)
  
;; Notes: I created this because I couldn't 'apply' or - it's an fsub, which
;; is pretty obstinate!

;; See Also: some

;; Examples:
 (or-function t t)
 (or-function 1 1)
 (or-function nil t)
 (or-function  nil nil)
 (apply #'or-function (list nil t))
 (mapcar #'or-function (list nil t))
 ;; but can't do this:
 ;; (apply #'or (list nil t))
"
  (if (some #'(lambda(x) x) x) t nil)
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                            eval-string
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun eval-string(x)
  "
;; Documentation: (eval-string string)

	Evaluates the first thing read from a string.

;; Examples:
 (def crud '(3 4 2))
 (def crud2 '(4 2))
 (eval-string \"crud\")
 (eval-string \"crud crud2\")
"
  (eval (read-from-string x))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                            remove-keys
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun remove-keys(keys from)
  "
;; Documentation: (remove-keys keys from)
 
	This will remove keys from a list (useful if you want to use the &rest
argument with keys).

;; Examples:
 (remove-keys '(:latex :print) '(1 2 :latex t)) 
 (remove-keys '(:latex :print) '(1 2 :latex t :print 'HI))
 (remove-keys '(:latex :print) '(1 2 :latex t :print 'HI :lazy 'yes))
"
  (dolist (i keys)
	  (let (
		(spot (grep (list i) from))
		)
	    (if spot
		(setq
		 from
		 (select
		  from
		  (remove (1+ spot) (remove spot (iseq (length from))))))
	      )
	    )
	  )
  from
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                            catch-keys
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun catch-keys(keys from)
  "
;; Documentation: (catch-keys keys from)
 
	This will catch key values from a list (useful if you want to use the
&rest argument with keys).

;; Examples:
 (catch-keys '(:latex :print) '(1 2 :latex t)) 
 (catch-keys '(:latex :print) '(1 2 :latex t :print 'HI))
 (catch-keys '(:latex :print) '(1 2 :latex t :print 'HI :lazy 'yes))
 (catch-keys '(:color :symbol :type :size)
   (list :color 'blue :symbol 'wedge1 :type 'dashed :size 2))
 (catch-keys '(:color :symbol :type :size)
   '(:color 'blue :symbol 'wedge1 :type 'dashed :size 2))
"
  (let (
	(keyvals (repeat nil (length keys)))
	(j 0)
	)
    (dolist (i keys)
	    (let (
		  (spot (grep (list i) from))
		  )
	      (if spot (setf (elt keyvals j) (elt from (1+ spot))))
	      (setq j (1+ j))
	      )
	    )
    keyvals
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 link-all-plots
;;
(defun link-all-plots()
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; link-all-plots Doc:    Creation Date:1/2/97        Author: Andy Long
;;
;; Description/Arguments:

       (link-all-plots)
  
;; Notes:

;; See Also:

;; Examples:
 (link-all-plots)
"
  (mapcar #'(lambda(x) (send x :linked t)) (agw))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 unlink-all-plots
;;
(defun unlink-all-plots()
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; unlink-all-plots Doc:    Creation Date:1/2/97        Author: Andy Long
;;
;; Description/Arguments:

       (unlink-all-plots)
  
;; Notes:

;; See Also:

;; Examples:
 (unlink-all-plots)
"
  (mapcar #'(lambda(x) (send x :linked nil)) (agw))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 greatest-divisor
;;
(defun greatest-divisor(n)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; greatest-divisor Doc:    Creation Date:1/2/97        Author: Andy Long
;;
;; Description/Arguments:

       (greatest-divisor n)
  
Produces the greatest integral divisor of integer n other than the number n
itself.

;; Notes:

;; See Also:

;; Examples:
 (greatest-divisor 18)
 ;; 9
 (greatest-divisor 1)
 ;; 1
"
  (let (
	stuff
	(x (reverse (iseq (floor (/ (+ 3 n) 2)))))
	)
    (tagbody
     (dolist
      (i x)
      (if (= (/ n i) (floor (/ n i)))
	  (let ()
	    (setq stuff i)
	    (go done)
	    )
	))
     done
     )
    stuff
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                select-if
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun select-if (a &optional (test #'(lambda(x) (= x 0))))
  "
;; Description:
		select-if (a &optional (test #'(lambda(x) (= x 0))))

	Note that the default is to keep zero elements.

;; Examples:
 (select-if '(0 1 2)) ;; (0)
 (select-if '(0 1 2) #'(lambda(x) (< x 2))) ;; (0 1)
"
  (let (keepers)
    (dolist
     (i a)
     (if (apply test (list i)) (setq keepers (append keepers (list i))))
     )
    keepers
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 select-from-end
;;
(defun select-from-end(x seq)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; select-from-end Doc:    Creation Date:1/3/97        Author: Andy Long
;;
;; Description/Arguments:

       (select-from-end list sequence)
  
	I created this mostly to choose the last several elements from the back
of a list.

;; Notes:

;; See Also:

;; Examples:
 (select-from-end (iseq 6) (iseq 3))
 ;; (3 4 5)
 (select-from-end (iseq 6) 3)
 ;; (3 4 5)
"
  (if (numberp seq)
      (reverse (select (reverse x) (iseq seq)))
    (reverse (select (reverse x) seq))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 snip-string
;;
(defun snip-string(st &key downcase)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; snip-string Doc:    Creation Date:1/21/97        Author: Andy Long
;;
;; Description/Arguments:

	snip-string (st &key downcase)
  
;; Notes:

;; See Also:

;; Examples:
 (snip-string \"intercept slope intercept-se slope-se\")
 (snip-string \"intercept slope intercept-se slope-se\" :downcase t)
"
 (if downcase
     (mapcar #'string-downcase (mapcar #'symbol-name (string-to-list st)))
   (mapcar #'symbol-name (string-to-list st))
   )
 )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 matrix-to-sas
;;
(defun matrix-to-sas(matrix variables name &key command)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; matrix-to-sas Doc:    Creation Date:1/29/97        Author: Andy Long
;;
;; Description/Arguments:

       (matrix-to-sas matrix variables name &key command)
  
	matrix:		matrix to write
	variables:	list of variables names as strings
	name:		string (name of the data and filename, although a .sas
			extension will be added for the filename)

	options:
	command		add a string which will be appended to the filename.
			This should contain valid sas commands.

Writes a matrix to a .sas file which sas can read.

;; Notes: the variable names should be valid sas variables (no more than eight
;; characters, avoiding things like '.' and the like.

;; Examples:
 (matrix-to-sas (id 2) (list \"hi\" \"lo\") \"tmpola\")
 ;; this will create file tmpola.sas, with two variables, hi and lo.
"
  (let (
	(f (open (strcat name ".sas") :direction :output))
	(shortname (select name (iseq (min (list 8 (length name))))))
	)
    (format f
	    "options ls=70 ps=50 nodate nonumber;~%data ~a;~%input" shortname)
    (mapcar #'(lambda(x) (format f "~%~a" x)) variables)
    (format f ";~%cards;~%")
    (dolist
     (i (row-list matrix))
     (mapcar #'(lambda(x) (format f "~g " x)) (coerce i 'list))
     (format f "~%")
     )     
    (format f ";~%proc contents data=~a;~%" shortname)

    (if command (format f "~a~%" command))
    (close f)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 where
;;
(defun where(variable value &optional (function #'equalp))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; where Doc:    Creation Date:2/3/97        Author: Andy Long
;;
;; Description/Arguments:

       (where variable value &optional (function #'equalp))
  
Optional:
	function
		The function should be some comparison function for each
		element of the list. The default is equality

	I wrote this function to return the places where a list has a certain
value. It serves as a preliminary result to use with select.

;; Notes:

;; See Also: simul-keep-if, select

;; Examples:
 (def sex (list 1 1 0 1 1 0 0 0 0))
 (def bp (list 90 100 80 71 112 60 87 74 109))

 (where sex 1)
 ;; (0 1 3 4): another way to say that is with the not-equal function:

 (where sex 0 #'/=)
 ;; (0 1 3 4)

 (where sex 0)
 ;; (2 5 6 7 8)

 (select bp (where sex 0))
 ;; (80 60 87 74 109)

 (select bp (where sex 0))
 ;; (80 60 87 74 109)
"
  (second
   (simul-keep-if
    (if (listp variable) variable (coerce variable 'list))
    (iseq (length variable))
    #'(lambda(x) (apply function (list x value)))
    )
   )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 ou
;;
(defmacro ou(&rest args)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; ou Doc:    Creation Date:2/17/97        Author: Andy Long
;;
;; Description/Arguments: macro

       (ou list function value)
  
	Where function(list)=value. This is just a more natural way of using
the where function. As usual, beware of using macros inside of functions.

;; Notes: ou means 'where' in French.

;; See Also: where

;; Examples:
 (ou (iseq 1 4) = 3)
"
  (let (
	(lis (first args))
	(fn (second args))
	(val (third args))
	)
    `(where ,lis ,val #',fn)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 indicator
;;
(defun indicator(list subset)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; indicator Doc:    Creation Date:2/27/97        Author: Andy Long
;;
;; Description/Arguments:

       (indicator list subset)
  
	Creates an indicator variable (1 for the subset indices, 0 elsewhere).

;; Notes:

;; See Also:

;; Examples:
 (def sex (list 0 0 1 1))
 (def cancer (list 1 0 1 0))
 (indicator cancer (where sex 1))
"
  (let (
	(tmp (make-array (length list) :initial-element 0))
	)
    (mapcar (lambda(x) (setf (aref tmp x) 1)) subset)
    (coerce tmp 'list)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                       space-print-num
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun space-print-num(x &optional (n 1))
  "Args: (space-print-num x &optional (n 1))
Prints a number after n spaces.
"
  (format t (strcat (apply #'strcat (repeat "~t" n)) "~g~%") x)
  )
;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                find-duplicates
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun find-duplicates(list &key verbose)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; find-duplicates Doc:    Creation Date:2/24/97        Author: Andy Long
;;
;; modifications: 11/15/97, goal being to speed this up! It's terribly slow as
;; previously defined in utility.lsp...ael
;;
;; Description/Arguments:

       (find-duplicates list &key verbose)
  
	find-duplicates will look over list for elements that are identical (in
the equalp sense). It returns a list of list of indices for identical items.

;; Notes: I wrote this to improve remove-duplicates, which currently only works
with numbers, other than lists.

;; See Also: equalp, remove-dups

;; Examples:
 (def stuff '(#(1 2) #(2 3) #(1 2)))
 (equalp (first stuff) (third stuff))
 (find-duplicates stuff)
 ;; ((0 2)

 (def stuff '(#(1 2) #(2 3) #(1 2) #(1 2) #(2 3) #(3 5)))
 (equalp (first stuff) (third stuff))
 (find-duplicates stuff)
 ;; ((0 2 3) (1 4))

 (remove-duplicates stuff)
 (remove-duplicates '(\"hi\" \"hi\" \"ho\"))
 (find-duplicates '(\"hi\" \"hi\" \"ho\"))
 ;; ((0 1))
"
  (let* (
	 (len (length list))
	 (len-1 (1- len))
	 (over (make-array len :initial-element t))
	 dups
	 )
    (mapcar
     #'(lambda(i)
	 (if (aref over i)
	     (let* (
		    (first (elt list i))
		    (list-first (list first))
		    (found
		     (search
		      list-first
		      (select list (iseq (1+ i) len-1))
		      :test 'equalp))
		    )
	       (if found
		   (let*
		       (
			(start (+ 1 i found))
			(stuff (list i start))
			)
		     (if verbose (print i))
		     (do
		      ()
		      (
		       (or (not found) (if found (= start len-1)))
		       (setq
			dups
			(append
			 dups
			 (let ()
			   (setf (aref over i) nil)
			   ;; (that leaves only unique elements t)
			   (list stuff)
			   )
			 )
			))
		      (setf (aref over start) nil)
		      (setq found
			    (search
			     list-first
			     (select list (iseq (1+ start) len-1))
			     :test 'equalp))
		      (if found
			  (setq start (+ 1 start found)
				stuff (if found (combine stuff start))))
		      )
		     )
		 )
	       )
	   )
	 )
     (iseq len-1)
     )
    (values dups (where over t))
    )
  )
;;
#|
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 remove-dups
;;
(defun remove-dups(list)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; remove-dups Doc:    Creation Date:2/26/97        Author: Andy Long
;;
;; Description/Arguments:

       (remove-dups list)
  
	Created to improve upon remove-duplicates.

;; Notes:

;; See Also: find-duplicates, remove-duplicates

;; Examples:
 (def stuff '(#(1 2) #(2 3) #(1 2) #(1 2) #(2 3) #(3 5)))
 (equalp (first stuff) (third stuff))
 (find-duplicates stuff)
 ;; ((0 2 3) (1 4))
 (remove-duplicates stuff)
 (remove-dups stuff)
 ;; (#(3 5) #(1 2) #(2 3))

 (remove-duplicates '(\"hi\" \"hi\" \"ho\"))
 (find-duplicates '(\"hi\" \"hi\" \"ho\"))
 ;; ((0 1))
 (remove-dups '(\"hi\" \"hi\" \"ho\"))
"
  (if (v-and (mapcar #'numberp list))
      ;; then we're in the old case, for which remove-duplicates works fine:
      (remove-duplicates list) ; otherwise, 
    (let* (
	   (dups (find-duplicates list))
	   (first-dups (mapcar #'first dups))
	   (duplicates (select list first-dups))
	   (non-duplicates
	    (select list (set-difference (iseq (length list)) (combine dups))))
	   )
      (append non-duplicates duplicates)
      )
    )
  )
;;
|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                remove-dups
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun remove-dups(list &key verbose)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; remove-dups Doc:    Creation Date: Mar 22, 1999  Author: Andy Long
;;
;; Description/Arguments:

	(remove-dups list &key verbose)
  
	remove-dups will look over list for elements that are identical
  (in the equalp sense).

;; Notes: I wrote this to speed up remove-dups.
;; See Also: equalp, remove-dups

;; Examples:
 ;; They work the same on numerical data:
 (remove-duplicates '(1 1 2 3 3))
 (remove-dups  '(1 1 2 3 3))

 ;; but when things get more complicated, remove-duplicates breaks down:
 (def stuff '(#(1 2) #(2 3) #(1 2)))
 (equalp (first stuff) (third stuff))
 (remove-dups stuff)
 ;; (#(1 2) #(2 3))

 (def stuff '(#(1 2) #(2 3) #(1 2) #(1 2) #(2 3) #(3 5)))
 (equalp (first stuff) (third stuff))

 (remove-duplicates stuff)
 ;; (#(1 2) #(2 3) #(1 2) #(1 2) #(2 3) #(3 5))
 (remove-dups stuff)
 ;; (#(1 2) #(2 3) #(3 5)) 

 (remove-duplicates '(\"hi\" \"hi\" \"ho\"))
 ;; (\"hi\" \"hi\" \"ho\")
 (remove-dups '(\"hi\" \"hi\" \"ho\"))
 ;; (\"hi\" \"ho\") 
"
  (let* ((dups (list (first list))))
    (dolist
     (item list)
     (if (not (search (list item) dups :test 'equalp))
	 (setq dups (append dups (list item)))))
    dups
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 gimme
;;
(defun gimme(&rest e)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; gimme Doc:    Creation Date:2/25/97        Author: Andy Long
;;
;; Description/Arguments:

       (gimme list &rest where-arguments
		&key
		(function #'intersection)
		)
  
	The idea is to allow for where expressions of the sorts shown in the
examples.

;; Notes:

;; See Also:

;; Examples:
 (def users (iseq 4))
 (def sex '(1 1 0 0))
 (def site '(1 0 1 0))
 (def smoking '(0 0 1 1))
 (gimme users (where sex 0))
 (gimme users (where site 0))
 (gimme users (where smoking 1))
 (gimme users (where sex 0) (where site 1) (where smoking 0 #'/=))

 ;; gimme anyone who is all of sex=0, site=1, or non-smoking:
 (gimme users (ou sex = 0) (ou site = 1) (ou smoking /= 0))

 ;; gimme anyone who is any of sex=0, site=1, or non-smoking:
 (gimme users (ou sex = 0) (ou site = 1) (ou smoking /= 0) :function #'union)
"
  (let* (
	 (list (first e))
	 (int (rest e))
	 (result (first int))
	 
	 (function (first (catch-keys '(:function) e)))
	 (int (if function (remove-keys '(:function) int) int))

	 (function (if function function #'intersection))
	 )
    (dolist
     (i int)
     (setq result (apply function (list result i)))
     )
    (select list result)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 compile-function
;;
(defun compile-function(function)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; compile-function Doc:    Creation Date:3/31/97        Author: Andy Long
;;
;; Description/Arguments:

       (compile-function function)
  
	This function just calls compile, and spares you the necessity of
giving the lambda-expression for the function. The argument is the symbol-name
of a function (see the examples below).

;; See Also: compile

;; Examples:
 (defun f(x) (sin x))
 (time (def tmp (mapcar #'f (iseq 1000))))
 (compile 'f)
 (time (def tmp (mapcar #'f (iseq 1000))))
 (plot f)

 (defun g(x) (cos x))
 (time (def tmp (mapcar #'g (iseq 1000))))
 (compile 'g)
 (time (def tmp (mapcar #'g (iseq 1000))))
 (plot g)
"
  (compile function (gle function))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 make-new-color
;;
(let ((Color-symbol-number 0))
  ;; Defun inside let form gives persistant variable associated
  ;; with function
  
  (defun make-new-color (r b g)
    "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          make-new-color Doc:
;;
;; Author: Andy Long
;; Created: Tue Apr 8 10:55:12 CDT 1997
;;
;; Description/Arguments:

       (make-new-color r b g)
  
Makes a new color with a unique name and returns the name. 

;; Notes:

;; See Also:

;; Examples:
 (plot)
 (make-new-color 1 0 0) ;; red
 (make-new-color 0 1 0) ;; green
 (make-new-color 0 0 1) ;; blue
 (make-new-color 1/3 1/3 1/3) ;; equal mix
 (send my-plot :add-plot-item
    (list 'points (rand.u 10) (rand.u 10) 'color-1)
    (list 'points (rand.u 10) (rand.u 10) 'color-2)
    (list 'points (rand.u 10) (rand.u 10) 'color-3)
    (list 'points (rand.u 10) (rand.u 10) 'color-4))
 (def update (send my-plot :update-plot))

 (draw-color 'color-1)
 (plot sin)
"
    (let (
	  (new-name
	   (intern (format nil "COLOR-~d" (incf color-symbol-number))))
	  )
      (make-color new-name r b g)
      new-name
      )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; grep Doc:    Creation Date:7/7/96        Author: Andy Long
;;
;;
(defun grep(exp object &key v test)
  "
;; Description/Arguments:

       (grep expression object test)
  
where object is a list or string, and expression is of the same type. Grep will
either tell you where an expression lives in an object.

	The default test is #'equal for strings, #'equalp for other things,
except for nan or not-a-numbers.

;; See Also: search

;; Examples:
 (grep '(1 1) '(1 2 3 1 1 5))
 (grep '(+ -) '(1 2 3 + - 5))
 (grep \"hi\" \"this\")
 ;; the previous command should return 1, since \"hi\" in the 1 spot of
 ;; \"this\".
 (grep \"hi\" \"ths\")
 ;; the previous command should return nil, since \"hi\" is not in \"ths\".

 ;; nan (not-a-number) is a bit of a problem, but I think that I've got it
 ;; licked:
 (grep (list nan) (list 1 2 nan))
 (grep (list #.not-a-number) (list 1 2 #.not-a-number))
 (def stuff (list 1 2 #.not-a-number))
 (grep (list #.not-a-number) stuff)
 (grep (list #.not-a-number) (/ stuff 10))
 (def stuff (list 1 2 nan))
 (grep (list nan) stuff)
 (grep \"/\" \"02/03/04\")
 (grep (list \"nig.men\") (list \"nig.men\"))
 (grep '(EXP X) '(EXP X))
"
  ;; This if is designed to handle the not-a-number situations:
  (let (
	(test (if test test
		(if (stringp exp) #'equal
		  (if (and (= (length exp) 1) (not-a-numberp (first exp)))
		      (lambda(x y) (not-a-numberp y))
		    #'equalp))))
	)
    (search exp object :test test)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; grep-replace Doc:    Creation Date:7/7/96        Author: Andy Long
;;
;; Modified 1/2/97, by ael
;;
(defun grep-replace(badexp goodexp object)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; grep-replace Doc:    Creation Date:7/7/96        Author: Andy Long
;;
;; Modified 1/2/97, by ael
;;
;; Description/Arguments:

       (grep-replace bad-expression good-expression object)
  
where object is a list or string, and expression is of the same type.

;; See Also: search, grep

;; Examples:
 (grep-replace '(1 1) '(a) '(1 2 3 1 1 5))
 (grep-replace '(1 1) '(a) '(1 2 3 1 1 5 3 1 1 5))
 (grep-replace '(+ -) '(a) '(1 2 + - 1 5))
 (grep-replace '(+ -) '(a) '(1 2 + 1 1 5))
 (grep-replace \"hi\" \"HI\" \"this\")
 (grep-replace \"hi\" \"HI\" \"this his terrible!\")
 (grep-replace (list nan) (list nil) (list 1 nan 2 nan)) ;; (1 NIL 2 NIL)
 (grep-replace \"/\" \" \" \"02/03/04\")
 (grep-replace \",\" \" , \" \"02,03,04\")
 (grep-replace \" @ \" \"@\" \"02 @ 03@04\")
 (grep-replace (list 1) (list 1 2) (list 1 2 1)) ;; (1 2 2 1 2)
 (grep-replace '(1) '((1 2)) (list 1 2 1)) ;; ((1 2) 2 (1 2))
 (grep-replace '((1 2)) 1 (list '(1 2) 1)) ;; (1 1)
"
  (if (and (or (stringp goodexp) (listp goodexp)) (grep badexp goodexp))
      (let* (
	     (gobbltigook (if (stringp badexp) "@@@@@@@" (list 'aaaaaaa)))
	     (gobbltigook
	      (if (grep badexp gobbltigook)
		  (if (stringp badexp) "~~~~~~~" (list 'bbbbbbb))
		gobbltigook
		)
	      )
	     )
;; Tried to do an end run, but failed: the thing you want replaced
;; should not appear in the thing with which you wish to replace it: this would
;; lead to an infinite loop. 
	(setq object (grep-replace badexp gobbltigook object))
	(grep-replace gobbltigook goodexp object)
	)
    (let ((len (length badexp)))
      (if (stringp object)
	  (do
	   ((in (grep badexp object) (grep badexp object)))
	   ((not in))
	   (setf object
		 (strcat
		  (subseq object 0 in)
		  goodexp
		  (subseq object (+ in len) (length object))
		  )
		 )	   
	   )
	(do
	 ((in (grep badexp object) (grep badexp object)))
	 ((not in))
	 (if (/= 0 in)
	     (if (= (1- (length object)) in)
		 (setq object
		       (append
			(butlast object)
			(if (listp goodexp) goodexp (list goodexp))
			))
	       (setq object
		     (append
		      (select object (iseq 0 (1- in)))
		      (if (listp goodexp) goodexp (list goodexp))
		      (select object (iseq (+ in len) (1- (length object))))
		      )))
	   (setq object (append (if (listp goodexp) goodexp (list goodexp))
				(rest object)))
	   )
	 )
	)
      object
      )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; recursive-grep-replace Doc:    Creation Date://        Author: Andy Long
;;
;;
(defun recursive-grep-replace(badexp goodexp object)
  "
;; Description/Arguments:

       (recursive-grep-replace bad-expression good-expression object)
  
;; Notes:

;; See Also: grep, grep-replace

;; Examples:
 (recursive-grep-replace '(* -) '(* -1 *) '((SIN (- 5) * - 6)))
 (recursive-grep-replace '(1) '(2) '((1) 1 (2 1 3)))
 (recursive-grep-replace '(nan) '(nil) '((nan) 1 nan 2 (2 nan 3)))
 (recursive-grep-replace '(EXP X) '(3) '(EXP X))
 (recursive-grep-replace '(EXP X) 3 '(EXP X))
"
  (if (equalp badexp object) (setq object goodexp)
    (if (listp object)
	(let()
	  (dotimes
	   (i (length object))
	   (if (listp (elt object i))
	       (setf (elt object i)
		     (recursive-grep-replace badexp goodexp (elt object i)))
	     )
	   )
	  (setf object (grep-replace badexp goodexp object))
	  )
      (if (and (stringp object) (stringp badexp))
	  (setf object (grep-replace badexp goodexp object))
	)
      )
    )
  object
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; recursive-grep Doc:    Creation Date:2/22/97        Author: Andy Long
;;
;;
(defun recursive-grep(goodexp object)
  "
;; Description/Arguments:

       (recursive-grep bad-expression good-expression object)
  
;; Notes:

;; See Also: grep, grep-replace

;; Examples:
 (recursive-grep '(* -)  '((SIN (- 5) * - 6)))
 (recursive-grep '(1) '((1) 1 (2 1 3)))
 (recursive-grep '(EXP X) '(EXP X))
"
  (if (equalp goodexp object) t
    (let (stuff)
      (if (listp object)
	  (dolist
	   (i object)
	   (setq stuff
		 (append stuff
			 (list 
			  (if (listp i)
			      (recursive-grep goodexp i)
			    (grep goodexp (list i))
			    )
			  ))))
	)
      stuff
      )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 remove-bunch
;;
(defun remove-bunch(bunch list &key (test #'equalp))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          remove-bunch Doc:
;;
;; Author: Andy Long
;; Created: Thu May 1 14:55:28 CDT 1997
;;
;; Description/Arguments:

       (remove-bunch bunch list &key (test #'equalp))
  
;; Notes:

;; See Also:

;; Examples:
 (remove-bunch (list \"ho\") (list \"hi\" \"ho\" \"hie\" \"hee\"))
 (remove-bunch (list \"hi\" \"ho\") (list \"hi\" \"ho\" \"hie\" \"hee\"))
"
  (let (
	(test (if test test #'equalp))
	(result list)
	)
    (dolist
     (i bunch)
     (setq result (remove i result :test test))
     )
    result
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 find-min
;;
(defun find-min(x &key (test #'min))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          find-min Doc:
;;
;; Author: Andy Long
;; Created: Thu May 15 13:07:32 CDT 1997
;;
;; Description/Arguments:

       (find-min x &key (test #'min))
  
;; Notes:

;; See Also:

;; Examples:
 (find-min (iseq 3))
 (find-min (iseq 3) :test #'max)
"
  (search (list (find (funcall test x) x)) x)
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 find-max
;;
(defun find-max(x &key (test #'max))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          find-max Doc:
;;
;; Author: Andy Long
;; Created: Thu May 15 13:07:32 CDT 1997
;;
;; Description/Arguments:

       (find-max x &key (test #'max))
  
;; Notes:

;; See Also:

;; Examples:
 (find-max (iseq 3))
 (find-max (iseq 3) :test #'min)
"
  (find-min x :test test)
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                By
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun by (analysis variables by-variables)
  "
;; Description: function (by analysis variables by-variables)

\"...a function for performing an analysis separately for each
combination of levels of a set of variables.  For example, you could get
means or other summaries of income by sex and race, or perform a
regression analysis separately for each county or state in a database.

I'm not sure where this should go in your archive but I think people
should find it useful.

thomas lumley, Biostatistics, Uni of Washington\"

	Args:  analysis  variables by-variables.

	For each combination of levels of by-variables, perform analysis on
that subset of the specified variables. Returns a list of the results.

  eg (by '(gee-model :x x :y y :g g :error normal-error :verbose t)
                 '(x y g) (list sex race))

     (by '(gee-model :x (list x1 x2 x3) :y y :g g :error normal-error 
                  :verbose t :predictor-names (list \"x1\" \"x2\" \"x3\"))
                 '(x1 x2 x3 y g) (list sex race))

     (by '(mean age) '(age) (list smoker gender))

     (by '(plot-points x y) '(x y) (list state))

  The variables must all describe data of the same length, they
 can be sequences or  matrices

 by-variables is a list of sequences.

;; Examples:
 (def x (rseq -pi pi 100))
 (def y (sin x))
 (def state (signum y))
 ;; This should plot the negatives versus the positives
 (by '(plot-points x y) '(x y) (list state))
"
  (flet (
	 (make-rows(data)
		   (cond ((matrixp data) (row-list data))
			 ((sequencep data) data)
			 (t (error "Can't convert this to rows"))
			 )
		   )
	 (equal-sequence (x y) (mapcar #'(lambda (z) (equalp z y)) x))
	 )
	(let* (
	       (byrows (row-list (apply #'bind-columns by-variables)))
	       (unique-rows (remove-duplicates byrows :test #'equalp))
	       (byindex
		(mapcar
		 (lambda(x)
		   (which
		    (equal-sequence
		     (row-list (apply #'bind-columns by-variables)) x)))
		 unique-rows))
	       (setup
		(mapcar 
		 (lambda(vv)
		   (if (matrixp (eval vv))
		       `(,vv (select ,vv indices
				     (iseq (second (array-dimensions ,vv)))))
		     `(,vv (select ,vv indices))))
		 variables))
	       )
	  (eval
	   `(mapcar #'(lambda (indices) (let* ,setup ,analysis)) ',byindex))
	  )
	)
  )

;; add to manual: sort-symbols
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				sort-symbols
;;
(defun sort-symbols(x &optional (test #'string<))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			sort-symbols Doc:
;;
;; Author: Andy Long
;; Created: Sun Apr 26 11:10:23 1998
;;
;; Description/Arguments: sort-symbols

	Sorts a list of symbols by mapping them to strings, sorting the strings
 (with #'string< by default), then writing them as a list of symbols again.

;; Arguments: list &optional (test #'string<)

	list	- a list of symbols
	test	- a function for sorting a list of strings

;; Returns: the sorted list of symbols

;; See Also: sort, read-from-string

;; Examples:
 (sort-symbols (send *object* :slot-names))
 ;; in descending alphabetic order:
 (sort-symbols (send *object* :slot-names) #'string>)
"
  (mapcar #'read-from-string (sort (mapcar #'symbol-name x) test)))
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun read-matrix-data-with-one-line-header(file)
  (let* (
	  (a (read-matrix-data file))
	   (matrix (but-rows a 0))
	    (name (row 1 a))
	     )
    ;;    `(mvsq (row 1 ,a) (column-list ,matrix))
    (mvsq name (column-list matrix))
    (values matrix name)
    )
  )
(defun read-csv-data-with-one-line-header(file)
  (let* (
	  (a (csv-import file))
	   (matrix (but-rows a 0))
	    (name (row 1 a))
	     )
    (mvsq name (column-list matrix))
    (values matrix name)
    )
  )
